package fyloz.trial.ColorRecipesExplorer.web.controller.creators;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Company;
import fyloz.trial.ColorRecipesExplorer.services.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_COMPANY;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_COMPANY_SUCCESS;

@Controller
public class CompanyCreatorController {

    private CompanyService companyService;

    @Autowired
    public CompanyCreatorController(CompanyService companyService) {
        this.companyService = companyService;
    }

    /**
     * Affiche la page de création d'une bannière.
     *
     * @return La page à afficher.
     */
    @GetMapping(CREATOR_COMPANY)
    public ModelAndView showCreationPage(ModelAndView model, Company company) {
        return new ModelBuilder(model)
                .setView(CREATOR_COMPANY)
                .addData(ModelDataType.COMPANY, company)
                .build();
    }

    /**
     * Permet à l'utilisateur de créer un bannière.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à effectuer cette action
     * - La bannière existe déjà
     * - Une erreur est survenue lors de la sauvegarde dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - companyName: Contient le nom de la bannière
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param company La bannière à créer
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_COMPANY, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView createCompany(@ModelAttribute @Valid Company company) {
        ModelBuilder modelBuilder = new ModelBuilder(CREATOR_COMPANY_SUCCESS);

        if (companyService.isValidForCreation(company)) {
            if ((company = companyService.save(company)) != null) {
                return modelBuilder
                        .addData(ModelDataType.COMPANY_NAME, company.getCompanyName())
                        .build();
            } else {
                modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING);
            }
        } else {
            modelBuilder.addResponseCode(ResponseCode.MIX_NOT_ASSOCIATED_WITH_RECIPE, company.getCompanyName(), null);
        }

        return showCreationPage(modelBuilder.build(), company);
    }
}
