package fyloz.trial.ColorRecipesExplorer.web.controller.creators;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Material;
import fyloz.trial.ColorRecipesExplorer.services.MaterialService;
import fyloz.trial.ColorRecipesExplorer.services.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_MATERIAL;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_MATERIAL_SUCCESS;

@Controller
public class MaterialCreatorController {

    private MaterialService materialService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialCreatorController(MaterialService materialService, MaterialTypeService materialTypeService) {
        this.materialService = materialService;
        this.materialTypeService = materialTypeService;
    }

    /**
     * Affiche la page de création d'un produit.
     *
     * @return La page à afficher.
     */
    @GetMapping(CREATOR_MATERIAL)
    public ModelAndView showCreationPage(ModelAndView model, Material material) {
        return new ModelBuilder(model)
                .setView(CREATOR_MATERIAL)
                .addData(ModelDataType.MATERIAL, material != null ? material : new Material())
                .addData(ModelDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .build();
    }

    /**
     * Permet à l'utilisateur de créer un produit.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à effectuer cette action
     * - Le produit existe déjà
     * - Une erreur est survenue lors de la sauvegarde dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - materialCode: Contient le code de produit du produit créé
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param material Le produit à créer
     * @param simdut   Le fichier SIMDUT du produit (optionnel)
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_MATERIAL, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ModelAndView create(@Valid Material material, MultipartFile simdut) {
        ModelBuilder modelBuilder = new ModelBuilder(CREATOR_MATERIAL_SUCCESS);

        if (!materialService.exists(material)) {
            if ((material = materialService.save(material)) != null) {
                modelBuilder.addData(ModelDataType.MATERIAL_CODE, material.getMaterialCode());

                if (!materialService.addSimdut(simdut, material)) {
                    modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING_SIMDUT);
                }

                return modelBuilder.build();
            } else {
                modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING);
            }
        } else {
            modelBuilder.addResponseCode(ResponseCode.MATERIAL_ALREADY_EXIST, material.getMaterialCode());
        }

        return showCreationPage(modelBuilder.build(), material);
    }
}
