package fyloz.trial.ColorRecipesExplorer.web.controller.creators;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.MaterialType;
import fyloz.trial.ColorRecipesExplorer.services.MaterialTypeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;

import static fyloz.trial.ColorRecipesExplorer.core.utils.ControllerUtils.redirect;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_MATERIAL_TYPE;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.INDEX;

@Controller
public class MaterialTypeCreatorController {

    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialTypeCreatorController(MaterialTypeService materialTypeService) {
        this.materialTypeService = materialTypeService;
    }

    @GetMapping(CREATOR_MATERIAL_TYPE)
    public ModelAndView showPage(ModelAndView model, MaterialType materialType) {
        return new ModelBuilder(model)
                .setView(CREATOR_MATERIAL_TYPE)
                .addData(ModelDataType.MATERIAL_TYPE, materialType == null ? new MaterialType() : materialType)
                .build();
    }

    @PostMapping(CREATOR_MATERIAL_TYPE)
    public ModelAndView create(@Valid MaterialType materialType) {
        ModelBuilder modelBuilder = new ModelBuilder(redirect(INDEX));

        if (materialTypeService.isValidForCreation(materialType)) {
            if (materialTypeService.save(materialType) != null) {
                return modelBuilder.build();
            } else {
                modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING);
            }
        } else {
            modelBuilder.addResponseCode(ResponseCode.MIX_TYPE_ALREADY_USED, materialType.getMaterialTypeName());
        }

        return showPage(modelBuilder.build(), materialType);
    }
}
