package fyloz.trial.ColorRecipesExplorer.web.controller.creators;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Recipe;
import fyloz.trial.ColorRecipesExplorer.services.CompanyService;
import fyloz.trial.ColorRecipesExplorer.services.RecipeService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import javax.validation.Valid;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_RECIPE;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.CREATOR_RECIPE_SUCCESS;

@Controller
public class RecipeCreatorController {

    private RecipeService recipeService;
    private CompanyService companyService;

    @Autowired
    public RecipeCreatorController(RecipeService recipeService, CompanyService companyService) {
        this.recipeService = recipeService;
        this.companyService = companyService;
    }

    /**
     * Affiche la page de création d'une recette.
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(CREATOR_RECIPE)
    public ModelAndView showCreationPage(ModelAndView model, Recipe recipe) {
        return new ModelBuilder(model)
                .setView(CREATOR_RECIPE)
                .addData(ModelDataType.COMPANIES, companyService.getAll())
                .addData(ModelDataType.RECIPE, recipe == null ? new Recipe() : recipe)
                .build();
    }

    /**
     * Permet à l'utilisateur de créer une nouvelle recette.
     * <p>
     * La création échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Une erreur est survenue lors de la sauvegarde dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param recipe La recette à créer
     * @return La page à afficher.
     */
    @PostMapping(value = CREATOR_RECIPE)
    public ModelAndView createRecipe(@Valid Recipe recipe) {
        ModelBuilder modelBuilder = new ModelBuilder(CREATOR_RECIPE_SUCCESS);

        if ((recipe = recipeService.save(recipe)) != null) {
            return modelBuilder
                    .addData(ModelDataType.RECIPE, recipe)
                    .build();
        }
        modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING);

        return showCreationPage(modelBuilder.build(), recipe);
    }
}
