package fyloz.trial.ColorRecipesExplorer.web.controller.removers;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Company;
import fyloz.trial.ColorRecipesExplorer.services.CompanyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.REMOVER_COMPANY;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.REMOVER_COMPANY_SPECIFIC;

@Controller
public class CompanyRemoverController {

    private CompanyService companyService;

    @Autowired
    public CompanyRemoverController(CompanyService companyService) {
        this.companyService = companyService;
    }

    /**
     * Affiche la page listant toutes les bannières.
     * <p>
     * Modèle de la page:
     * - companies: Contient la liste de toutes les bannières
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher
     */
    @GetMapping(REMOVER_COMPANY)
    public ModelAndView showPage(ModelAndView model) {
        return new ModelBuilder(model)
                .setView(REMOVER_COMPANY)
                .addData(ModelDataType.COMPANIES, companyService.getAll())
                .build();
    }

    /**
     * Permet à l'utilisateur de supprimer une bannière.
     * La méthode requiert l'identifiant de la bannière à supprimer dans l'URL.
     * <p>
     * La suppression échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Il n'y a aucune bannière avec cet identifiant
     * - La bannière est associée à une ou plusieurs recettes
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - successCompanyName: Contient le nom de la bannière supprimée
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param companyID L'identifiant de la bannière
     * @return La page à afficher
     */

    @PostMapping(REMOVER_COMPANY_SPECIFIC)
    public ModelAndView removeCompany(@PathVariable int companyID) {
        ModelBuilder modelBuilder = new ModelBuilder("");
        Company company = companyService.getByID(companyID);

        if (companyService.exists(company)) {
            if (companyService.deleteIfNotLinked(company)) {
                modelBuilder.addData(ModelDataType.COMPANY_NAME, company.getCompanyName());
            } else {
                modelBuilder.addResponseCode(ResponseCode.COMPANY_LINKED, company.getCompanyName());
            }
        } else {
            modelBuilder.addResponseCode(ResponseCode.COMPANY_NOT_FOUND, String.valueOf(companyID));
        }

        return showPage(modelBuilder.build());
    }
}
