package fyloz.trial.ColorRecipesExplorer.web.controller.removers;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Material;
import fyloz.trial.ColorRecipesExplorer.services.MaterialService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import java.util.stream.Collectors;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.REMOVER_MATERIAL;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.REMOVER_MATERIAL_SPECIFIC;

@Controller
public class MaterialRemoverController {

    private MaterialService materialService;

    @Autowired
    public MaterialRemoverController(MaterialService materialService) {
        this.materialService = materialService;
    }

    /**
     * Affiche la page de suppression des produits
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(REMOVER_MATERIAL)
    public ModelAndView showPage(ModelAndView model) {
        return new ModelBuilder(model)
                .setView(REMOVER_MATERIAL)
                .addData(ModelDataType.MATERIALS, materialService.getAll().stream().filter(m -> !m.isMixType()).collect(Collectors.toList()))
                .build();
    }

    /**
     * Permet à l'utilisateur de supprimer un produit.
     * Cette méthode requiert l'identifiant du produit dans l'URL.
     * <p>
     * La suppression échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Le produit n'existe pas
     * - Le produit est lié à d'autres mélanges
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - materialCode: Contient le code de produit du produit supprimé
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param materialID L'identifiant du produit à supprimer
     * @return La page à afficher.
     */
    @PostMapping(REMOVER_MATERIAL_SPECIFIC)
    public ModelAndView removeMaterial(@PathVariable int materialID) {
        ModelBuilder modelBuilder = new ModelBuilder("");
        Material material = materialService.getByID(materialID);

        if (material == null) {
            modelBuilder.addResponseCode(ResponseCode.MIX_NOT_FOUND, String.valueOf(materialID));
        } else {
            if (materialService.deleteIfNotLinked(material)) {
                modelBuilder.addData(ModelDataType.MATERIAL_CODE, material.getMaterialCode());

                if (!materialService.removeSimdut(material)) {
                    modelBuilder.addResponseCode(ResponseCode.ERROR_SAVING_SIMDUT);
                }
            } else {
                modelBuilder.addResponseCode(ResponseCode.MATERIAL_LINKED, material.getMaterialCode());
            }
        }

        return showPage(modelBuilder.build());
    }
}
