package fyloz.trial.ColorRecipesExplorer.web.controller.removers;

import fyloz.trial.ColorRecipesExplorer.core.ModelBuilder;
import fyloz.trial.ColorRecipesExplorer.core.ModelDataType;
import fyloz.trial.ColorRecipesExplorer.core.ResponseCode;
import fyloz.trial.ColorRecipesExplorer.model.Recipe;
import fyloz.trial.ColorRecipesExplorer.services.RecipeService;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.ModelAndView;

import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.REMOVER_RECIPE;
import static fyloz.trial.ColorRecipesExplorer.web.PagesPaths.REMOVER_RECIPE_SPECIFIC;

@Controller
public class RecipeRemoverController {

    private RecipeService recipeService;

    public RecipeRemoverController(RecipeService recipeService) {
        this.recipeService = recipeService;
    }

    /**
     * Affiche la liste de toutes les recettes.
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(REMOVER_RECIPE)
    public ModelAndView listRecipes(ModelAndView model) {
        return new ModelBuilder(model)
                .setView(REMOVER_RECIPE)
                .addData(ModelDataType.RECIPES, recipeService.getRecipesByCompany())
                .build();
    }

    /**
     * Permet à l'utilisateur de supprimer une recette.
     * Cette méthode requiert l'identifiant de la recette dans l'URL.
     * <p>
     * La suppression échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - La recette n'existe pas
     * - Une erreur est survenue lors de la suppression dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipeCode: Contient la couleur de la recette
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param recipeID L'identifiant de la recette
     * @return La page à afficher.
     */
    @PostMapping(REMOVER_RECIPE_SPECIFIC)
    public ModelAndView removeRecipe(@PathVariable int recipeID) {
        ModelBuilder modelBuilder = new ModelBuilder("");
        Recipe recipe = recipeService.getByID(recipeID);

        // Affiche un erreur si le recette n'est pas trouvée
        if (recipe == null) {
            modelBuilder.addResponseCode(ResponseCode.RECIPE_NOT_FOUND, String.valueOf(recipeID));

            return listRecipes(modelBuilder.build());
        }

        // TODO erreur dans cette méthode
        recipeService.deleteRecipe(recipe);

        modelBuilder.addData(ModelDataType.RECIPE_CODE, recipe.getRecipeCode());
        return listRecipes(modelBuilder.build());
    }
}
